@@ -95,6 +95,13 @@ pub enum StdError {
         #[cfg(feature = "backtraces")]
         backtrace: Backtrace,
     },
+    #[error("Conversion error: ")]
+    ConversionOverflow {
+        #[from]
+        source: ConversionOverflowError,
+        #[cfg(feature = "backtraces")]
+        backtrace: Backtrace,
+    },
 }
 
 impl StdError {
@@ -376,6 +383,22 @@ impl PartialEq<StdError> for StdError {
                     false
                 }
             }
+            StdError::ConversionOverflow {
+                source,
+                #[cfg(feature = "backtraces")]
+                    backtrace: _,
+            } => {
+                if let StdError::ConversionOverflow {
+                    source: rhs_source,
+                    #[cfg(feature = "backtraces")]
+                        backtrace: _,
+                } = rhs
+                {
+                    source == rhs_source
+                } else {
+                    false
+                }
+            }
         }
     }
 }
@@ -429,6 +452,7 @@ pub enum OverflowOperation {
     Sub,
     Mul,
     Pow,
+    Shr,
 }
 
 impl fmt::Display for OverflowOperation {
@@ -446,7 +470,11 @@ pub struct OverflowError {
 }
 
 impl OverflowError {
-    pub fn new<U: ToString>(operation: OverflowOperation, operand1: U, operand2: U) -> Self {
+    pub fn new(
+        operation: OverflowOperation,
+        operand1: impl ToString,
+        operand2: impl ToString,
+    ) -> Self {
         Self {
             operation,
             operand1: operand1.to_string(),
@@ -455,6 +483,34 @@ impl OverflowError {
     }
 }
 
+/// The error returned by [`TryFrom`] conversions that overflow, for example
+/// when converting from [`Uint256`] to [`Uint128`].
+///
+/// [`TryFrom`]: std::convert::TryFrom
+/// [`Uint256`]: crate::Uint256
+/// [`Uint128`]: crate::Uint128
+#[derive(Error, Debug, PartialEq, Eq)]
+#[error("Error converting {source_type} to {target_type} for {value}")]
+pub struct ConversionOverflowError {
+    pub source_type: &'static str,
+    pub target_type: &'static str,
+    pub value: String,
+}
+
+impl ConversionOverflowError {
+    pub fn new(
+        source_type: &'static str,
+        target_type: &'static str,
+        value: impl Into<String>,
+    ) -> Self {
+        Self {
+            source_type,
+            target_type,
+            value: value.into(),
+        }
+    }
+}
+
 #[derive(Error, Debug, PartialEq, Eq)]
 #[error("Cannot devide {operand} by zero")]
 pub struct DivideByZeroError {
@@ -655,10 +711,11 @@ mod tests {
     fn implements_debug() {
         let error: StdError = StdError::from(OverflowError::new(OverflowOperation::Sub, 3, 5));
         let embedded = format!("Debug: {:?}", error);
-        assert_eq!(
-            embedded,
-            r#"Debug: Overflow { source: OverflowError { operation: Sub, operand1: "3", operand2: "5" } }"#
-        );
+        #[cfg(not(feature = "backtraces"))]
+        let expected = r#"Debug: Overflow { source: OverflowError { operation: Sub, operand1: "3", operand2: "5" } }"#;
+        #[cfg(feature = "backtraces")]
+        let expected = r#"Debug: Overflow { source: OverflowError { operation: Sub, operand1: "3", operand2: "5" }, backtrace: <disabled> }"#;
+        assert_eq!(embedded, expected);
     }
 
     #[test]
