@@ -2,10 +2,14 @@ mod decimal;
 mod fraction;
 mod isqrt;
 mod uint128;
+mod uint256;
+mod uint512;
 mod uint64;
 
 pub use decimal::Decimal;
 pub use fraction::Fraction;
 pub use isqrt::Isqrt;
 pub use uint128::Uint128;
+pub use uint256::Uint256;
+pub use uint512::Uint512;
 pub use uint64::Uint64;
