@@ -6,6 +6,7 @@ use std::iter::Sum;
 use std::ops;
 
 use crate::errors::{DivideByZeroError, OverflowError, OverflowOperation, StdError};
+use crate::{ConversionOverflowError, Uint256, Uint64};
 
 /// A thin wrapper around u128 that is using strings for JSON encoding/decoding,
 /// such that the full u128 range can be used for clients that convert JSON numbers to floats,
@@ -27,11 +28,11 @@ use crate::errors::{DivideByZeroError, OverflowError, OverflowOperation, StdErro
 /// assert_eq!(c.u128(), 70);
 /// ```
 #[derive(Copy, Clone, Default, Debug, PartialEq, Eq, PartialOrd, Ord, JsonSchema)]
-pub struct Uint128(
-    #[schemars(with = "String")] pub u128, // Simon thinks this should be private, but does not want to worry about breaking code right now
-);
+pub struct Uint128(#[schemars(with = "String")] u128);
 
 impl Uint128 {
+    pub const MAX: Self = Self(u128::MAX);
+
     /// Creates a Uint128(value).
     ///
     /// This method is less flexible than `from` but can be called in a const context.
@@ -133,6 +134,12 @@ impl Uint128 {
 // of the conflict with `TryFrom<&str>` as described here
 // https://stackoverflow.com/questions/63136970/how-do-i-work-around-the-upstream-crates-may-add-a-new-impl-of-trait-error
 
+impl From<Uint64> for Uint128 {
+    fn from(val: Uint64) -> Self {
+        val.u64().into()
+    }
+}
+
 impl From<u128> for Uint128 {
     fn from(val: u128) -> Self {
         Uint128(val)
@@ -163,6 +170,16 @@ impl From<u8> for Uint128 {
     }
 }
 
+impl TryFrom<Uint128> for Uint64 {
+    type Error = ConversionOverflowError;
+
+    fn try_from(value: Uint128) -> Result<Self, Self::Error> {
+        Ok(Uint64::new(value.0.try_into().map_err(|_| {
+            ConversionOverflowError::new("Uint128", "Uint64", value.to_string())
+        })?))
+    }
+}
+
 impl TryFrom<&str> for Uint128 {
     type Error = StdError;
 
@@ -208,6 +225,54 @@ impl<'a> ops::Add<&'a Uint128> for Uint128 {
     }
 }
 
+impl ops::Sub<Uint128> for Uint128 {
+    type Output = Self;
+
+    fn sub(self, rhs: Self) -> Self {
+        Uint128(self.u128().checked_sub(rhs.u128()).unwrap())
+    }
+}
+
+impl<'a> ops::Sub<&'a Uint128> for Uint128 {
+    type Output = Self;
+
+    fn sub(self, rhs: &'a Uint128) -> Self {
+        Uint128(self.u128().checked_sub(rhs.u128()).unwrap())
+    }
+}
+
+impl ops::Div<Uint128> for Uint128 {
+    type Output = Self;
+
+    fn div(self, rhs: Self) -> Self::Output {
+        Self(self.u128().checked_div(rhs.u128()).unwrap())
+    }
+}
+
+impl<'a> ops::Div<&'a Uint128> for Uint128 {
+    type Output = Self;
+
+    fn div(self, rhs: &'a Uint128) -> Self::Output {
+        Self(self.u128().checked_div(rhs.u128()).unwrap())
+    }
+}
+
+impl ops::Shr<u32> for Uint128 {
+    type Output = Self;
+
+    fn shr(self, rhs: u32) -> Self::Output {
+        Self(self.u128().checked_shr(rhs).unwrap())
+    }
+}
+
+impl<'a> ops::Shr<&'a u32> for Uint128 {
+    type Output = Self;
+
+    fn shr(self, rhs: &'a u32) -> Self::Output {
+        Self(self.u128().checked_shr(*rhs).unwrap())
+    }
+}
+
 impl ops::AddAssign<Uint128> for Uint128 {
     fn add_assign(&mut self, rhs: Uint128) {
         self.0 = self.0.checked_add(rhs.u128()).unwrap();
@@ -220,6 +285,42 @@ impl<'a> ops::AddAssign<&'a Uint128> for Uint128 {
     }
 }
 
+impl ops::SubAssign<Uint128> for Uint128 {
+    fn sub_assign(&mut self, rhs: Uint128) {
+        self.0 = self.0.checked_sub(rhs.u128()).unwrap();
+    }
+}
+
+impl<'a> ops::SubAssign<&'a Uint128> for Uint128 {
+    fn sub_assign(&mut self, rhs: &'a Uint128) {
+        self.0 = self.0.checked_sub(rhs.u128()).unwrap();
+    }
+}
+
+impl ops::DivAssign<Uint128> for Uint128 {
+    fn div_assign(&mut self, rhs: Self) {
+        self.0 = self.0.checked_div(rhs.u128()).unwrap();
+    }
+}
+
+impl<'a> ops::DivAssign<&'a Uint128> for Uint128 {
+    fn div_assign(&mut self, rhs: &'a Uint128) {
+        self.0 = self.0.checked_div(rhs.u128()).unwrap();
+    }
+}
+
+impl ops::ShrAssign<u32> for Uint128 {
+    fn shr_assign(&mut self, rhs: u32) {
+        self.0 = self.0.checked_shr(rhs).unwrap();
+    }
+}
+
+impl<'a> ops::ShrAssign<&'a u32> for Uint128 {
+    fn shr_assign(&mut self, rhs: &'a u32) {
+        self.0 = self.0.checked_shr(*rhs).unwrap();
+    }
+}
+
 impl Uint128 {
     /// Returns `self * numerator / denominator`
     pub fn multiply_ratio<A: Into<u128>, B: Into<u128>>(
@@ -232,15 +333,27 @@ impl Uint128 {
         if denominator == 0 {
             panic!("Denominator must not be zero");
         }
-        let val: u128 = (self.full_mul(numerator) / denominator)
+        (self.full_mul(numerator) / Uint256::from(denominator))
             .try_into()
-            .expect("multiplication overflow");
-        Uint128::from(val)
+            .expect("multiplication overflow")
     }
 
-    /// Multiplies two u128 values without overflow.
-    fn full_mul(self, rhs: impl Into<u128>) -> U256 {
-        U256::from(self.u128()) * U256::from(rhs.into())
+    /// Multiplies two u128 values without overflow, producing an
+    /// [`Uint256`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use cosmwasm_std::Uint128;
+    ///
+    /// let a = Uint128::MAX;
+    /// let result = a.full_mul(2u32);
+    /// assert_eq!(result.to_string(), "680564733841876926926749214863536422910");
+    /// ```
+    pub fn full_mul(self, rhs: impl Into<u128>) -> Uint256 {
+        Uint256::from(self.u128())
+            .checked_mul(Uint256::from(rhs.into()))
+            .unwrap()
     }
 }
 
@@ -296,19 +409,6 @@ impl<'a> Sum<&'a Uint128> for Uint128 {
     }
 }
 
-/// This module is purely a workaround that lets us ignore lints for all the code
-/// the `construct_uint!` macro generates.
-#[allow(clippy::all)]
-mod uints {
-    uint::construct_uint! {
-        pub struct U256(4);
-    }
-}
-
-/// Only used internally - namely to store the intermediate result of
-/// multiplying two 128-bit uints.
-use uints::U256;
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -362,11 +462,11 @@ mod tests {
 
     #[test]
     fn uint128_is_zero_works() {
-        assert_eq!(Uint128::zero().is_zero(), true);
-        assert_eq!(Uint128(0).is_zero(), true);
+        assert!(Uint128::zero().is_zero());
+        assert!(Uint128(0).is_zero());
 
-        assert_eq!(Uint128(1).is_zero(), false);
-        assert_eq!(Uint128(123).is_zero(), false);
+        assert!(!Uint128(1).is_zero());
+        assert!(!Uint128(123).is_zero());
     }
 
     #[test]
@@ -399,7 +499,8 @@ mod tests {
         assert_eq!(a + &b, Uint128(35801));
 
         // test - with owned and reference right hand side
-        assert_eq!((b.checked_sub(a)).unwrap(), Uint128(11111));
+        assert_eq!(b - a, Uint128(11111));
+        assert_eq!(b - &a, Uint128(11111));
 
         // test += with owned and reference right hand side
         let mut c = Uint128(300000);
@@ -409,6 +510,14 @@ mod tests {
         d += &b;
         assert_eq!(d, Uint128(323456));
 
+        // test -= with owned and reference right hand side
+        let mut c = Uint128(300000);
+        c -= b;
+        assert_eq!(c, Uint128(276544));
+        let mut d = Uint128(300000);
+        d -= &b;
+        assert_eq!(d, Uint128(276544));
+
         // error result on underflow (- would produce negative result)
         let underflow_result = a.checked_sub(b);
         let OverflowError {
@@ -419,12 +528,18 @@ mod tests {
 
     #[test]
     #[should_panic]
-    fn uint128_math_overflow_panics() {
+    fn uint128_add_overflow_panics() {
         // almost_max is 2^128 - 10
         let almost_max = Uint128(340282366920938463463374607431768211446);
         let _ = almost_max + Uint128(12);
     }
 
+    #[test]
+    #[should_panic]
+    fn uint128_sub_overflow_panics() {
+        let _ = Uint128(1) - Uint128(2);
+    }
+
     #[test]
     fn uint128_multiply_ratio_works() {
         let base = Uint128(500);
